<?php

if (!prgsession_is_registered('text_loaded')) {
  $text_loaded = false;
  prgsession_register('text_loaded',$text_loaded);
}

if (!prgsession_is_registered('text_translateStrings')) {
  $text_translateStrings = array();
  prgsession_register('text_translateStrings',$text_translateStrings);
}

if (!prgsession_is_registered('text_loaded_language')) {
  $text_loaded_language = '';
  prgsession_register('text_loaded_language',$text_loaded_language);
}

if (!prgsession_is_registered('text_short_circuit')) {
  $text_short_circuit = false;
  prgsession_register('text_short_circuit',$text_short_circuit);
}

if (!prgsession_is_registered('text_language')) {
  $text_language = '';
  prgsession_register('text_language',$text_language);
}


function text_load_strings() {
    global $text_language, $text_translateStrings, $text_loaded, $text_loaded_language, $text_short_circuit;

    
    $text_translateStrings = array();    
    $text_short_circuit = false;  /* initialization */
    
    $filename = PRG_PATH.PRG_LANGDIR.$text_language.'.po';
    if (!$text_language)
      return;
    $file = @fopen($filename, 'r');
    if ($file == false) {
        /* Uh-ho -- we can't load the file.  Just fake it.  :-)
           This is also for English, which doesn't use translations */
        $text_loaded = true;
        $text_loaded_language = $text_language;
        /* Avoid fuzzy matching when we didn't load strings */
        $text_short_circuit = true;  
        return;
    }
    
    $key = '';
    $SkipRead = false;
    while (! feof($file)) {
        if (! $SkipRead) {
            $line = trim(fgets($file, 4096));
        } else {
            $SkipRead = false;
        }
        
        if (preg_match('#^msgid "(.*)"$#i', $line, $match)) {
            if ($match[1] == '') {
                /*
                 * Potential multi-line
                 * msgid ""
                 * "string string "
                 * "string string"
                 */
                $key = '';
                $line = trim(fgets($file, 4096));
                while (preg_match('#^[ ]*"(.*)"[ ]*$#', $line, $match)) {
                    $key .= $match[1];
                    $line = trim(fgets($file, 4096));
                }
                $SkipRead = true;
            } else {
                /* msgid "string string" */
                $key = $match[1];
            }
        } elseif (preg_match('#^msgstr "(.*)"$#i', $line, $match)) {
            if ($match[1] == '') {
                /*
                 * Potential multi-line
                 * msgstr ""
                 * "string string "
                 * "string string"
                 */
                $text_translateStrings[$key] = '';
                $line = trim(fgets($file, 4096));
                while (preg_match('#^[ ]*"(.*)"[ ]*$#', $line, $match)) {
                    $text_translateStrings[$key] .= $match[1];
                    $line = trim(fgets($file, 4096));
                }
                $SkipRead = true;
            } else {
                /* msgstr "string string" */
                $text_translateStrings[$key] = $match[1];
            }
#            $text_translateStrings[$key] =
#                stripslashes($text_translateStrings[$key]);
            /* If there is no translation, just use the untranslated string */
            if ($text_translateStrings[$key] == '') {
                $text_translateStrings[$key] = $key;
            }
            $key = '';
        }
    }
    fclose($file);
    
    $text_loaded = true;
    $text_loaded_language = $text_language;
}

function x($str) {
    global $text_loaded, $text_translateStrings, 
        $text_language, $text_loaded_language,
        $text_short_circuit;
  
    $args=func_get_args();
    $narg=func_num_args()-1;
    unset($args[0]);

    if (!$text_language) {
      if ($narg)
        return vsprintf($str,$args);
      return $str;
    }

    if (!$text_loaded || $text_loaded_language != $text_language)
        text_load_strings();

    /* Try finding the exact string */
    if (isset($text_translateStrings[$str])) {
      if ($narg) 
        return vsprintf($text_translateStrings[$str],$args);
      return $text_translateStrings[$str];
    }
    
    /* See if we should short-circuit */
    if ($text_short_circuit) {
        $text_translateStrings[$str] = $str;
	if ($narg)
          return vsprintf($str,$args);
        return $str;
    }
    
    /* Look for a string that is very close to the one we want
       Very computationally expensive */
    $oldPercent = 0;
    $oldStr = '';
    $newPercent = 0;
    foreach ($text_translateStrings as $k => $v) {
        similar_text($str, $k, $newPercent);
        if ($newPercent > $oldPercent) {
            $oldStr = $v;
            $oldPercent = $newPercent;
        }
    }
    /* Require 95% match or better
       Adjust to suit your needs */
    if ($oldPercent >= 95) {
        /* Remember this so we don't need to search again */
        $text_translateStrings[$str] = $oldStr;
	if ($narg)
          return vsprintf($oldStr,$args);	
        return $oldStr;
    }
    
    /* Remember this so we don't need to search again */
    $text_translateStrings[$str] = $str;
    if ($narg)
      return vsprintf($str,$args);
    return $str;
}



function text_setlanguage($lang) {
  global $text_language;
  $text_language = $lang;
  text_load_strings();
}

?>